@@ -9,8 +9,8 @@
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import, unicode_literals
-from builtins import *
 from . import mat_documenters as doc
+from . import mat_directives
 
 import re
 
@@ -36,6 +36,15 @@ mat_sig_re = re.compile(
           ''', re.VERBOSE)
 
 
+def strip_package_prefix(varname):
+    """Remove the leading '+' prefix on package names"""
+
+    if not varname:
+        return varname
+
+    return '.'.join([s.lstrip('+') for s in varname.split('.')])
+
+
 def _pseudo_parse_arglist(signode, arglist):
     """"Parse" a list of arguments separated by commas.
 
@@ -141,6 +150,12 @@ class MatObject(ObjectDescription):
         # determine module and class name (if applicable), as well as full name
         modname = self.options.get(
             'module', self.env.temp_data.get('mat:module'))
+
+        if not self.env.config.matlab_keep_package_prefix:
+            modname = strip_package_prefix(modname)
+            name_prefix = strip_package_prefix(name_prefix)
+            name = strip_package_prefix(name)
+
         classname = self.env.temp_data.get('mat:class')
         if classname:
             add_module = False
@@ -180,6 +195,8 @@ class MatObject(ObjectDescription):
             modname = self.options.get(
                 'module', self.env.temp_data.get('mat:module'))
             if modname and modname != 'exceptions':
+                if not self.env.config.matlab_keep_package_prefix:
+                    modname = strip_package_prefix(modname)
                 nodetext = modname + '.'
                 signode += addnodes.desc_addname(nodetext, nodetext)
 
@@ -211,9 +228,17 @@ class MatObject(ObjectDescription):
         modname = self.options.get(
             'module', self.env.temp_data.get('mat:module'))
         fullname = (modname and modname + '.' or '') + name_cls[0]
+
+        if not self.env.config.matlab_keep_package_prefix:
+            modname_out = strip_package_prefix(modname)
+            fullname_out = (modname_out
+                    and modname_out + '.' or '') + name_cls[0]
+        else:
+            modname_out, fullname_out = modname, fullname
+
         # note target
         if fullname not in self.state.document.ids:
-            signode['names'].append(fullname)
+            signode['names'].append(fullname_out)
             signode['ids'].append(fullname)
             signode['first'] = (not self.names)
             self.state.document.note_explicit_target(signode)
@@ -227,7 +252,7 @@ class MatObject(ObjectDescription):
                     line=self.lineno)
             objects[fullname] = (self.env.docname, self.objtype)
 
-        indextext = self.get_index_text(modname, name_cls)
+        indextext = self.get_index_text(modname_out, name_cls)
         if indextext:
             entry = ('single', indextext, fullname, '', None)
             self.indexnode['entries'].append(entry)
@@ -243,7 +268,7 @@ class MatObject(ObjectDescription):
 
 class MatModulelevel(MatObject):
     """
-    Description of an object on module level (functions, data).
+    Description of an object on module level (functions, data, application).
     """
 
     def needs_arglist(self):
@@ -258,6 +283,10 @@ class MatModulelevel(MatObject):
             if not modname:
                 return _('%s (built-in variable)') % name_cls[0]
             return _('%s (in module %s)') % (name_cls[0], modname)
+        elif self.objtype == 'application':
+            if not modname:
+                return _('%s (built-in application)') % name_cls[0]
+            return _('%s (in module %s)') % (name_cls[0], modname)
         else:
             return ''
 
@@ -417,6 +446,12 @@ class MatModule(Directive):
     def run(self):
         env = self.state.document.settings.env
         modname = self.arguments[0].strip()
+
+        if not env.config.matlab_keep_package_prefix:
+            modname_out = strip_package_prefix(modname)
+        else:
+            modname_out = modname
+
         noindex = 'noindex' in self.options
         env.temp_data['mat:module'] = modname
         ret = []
@@ -433,7 +468,7 @@ class MatModule(Directive):
             # the platform and synopsis aren't printed; in fact, they are only
             # used in the modindex currently
             ret.append(targetnode)
-            indextext = _('%s (module)') % modname
+            indextext = _('%s (module)') % modname_out
             entry = ('single', indextext, 'module-' + modname, '', None)
             inode = addnodes.index(entries=[entry])
             ret.append(inode)
@@ -468,7 +503,7 @@ class MatXRefRole(XRefRole):
         refnode['mat:class'] = env.temp_data.get('mat:class')
         if not has_explicit_title:
             title = title.lstrip('.')   # only has a meaning for the target
-            target = target.lstrip('~') # only has a meaning for the title
+            target = target.lstrip('~')  # only has a meaning for the title
             # if the first character is a tilde, don't display the module/class
             # parts of the contents
             if title[0:1] == '~':
@@ -476,6 +511,10 @@ class MatXRefRole(XRefRole):
                 dot = title.rfind('.')
                 if dot != -1:
                     title = title[dot+1:]
+
+            if not env.config.matlab_keep_package_prefix:
+                title = strip_package_prefix(title)
+
         # if the first character is a dot, search more specific namespaces first
         # else search builtins first
         if target[0:1] == '.':
@@ -491,7 +530,7 @@ class MATLABModuleIndex(Index):
 
     name = 'modindex'
     localname = _('MATLAB Module Index')
-    shortname = _('modules')
+    shortname = _('matlab index')
 
     def generate(self, docnames=None):
         content = {}
@@ -520,7 +559,13 @@ class MATLABModuleIndex(Index):
             if not modname:
                 modname, stripped = stripped, ''
 
-            entries = content.setdefault(modname[0].lower(), [])
+            # Create nice mod-name
+            if not self.domain.env.config.matlab_keep_package_prefix:
+                modname_out = strip_package_prefix(modname)
+            else:
+                modname_out = modname
+
+            entries = content.setdefault(modname_out[0].lower(), [])
 
             package = modname.split('.')[0]
             if package != modname:
@@ -538,7 +583,7 @@ class MATLABModuleIndex(Index):
                 subtype = 0
 
             qualifier = deprecated and _('Deprecated') or ''
-            entries.append([stripped + modname, subtype, docname,
+            entries.append([stripped + modname_out, subtype, docname,
                             'module-' + stripped + modname, platforms,
                             qualifier, synopsis])
             prev_modname = modname
@@ -569,6 +614,7 @@ class MATLABDomain(Domain):
         'attribute':    ObjType(_('attribute'),     'attr', 'obj'),
         'module':       ObjType(_('module'),        'mod', 'obj'),
         'script':       ObjType(_('script'),        'scpt', 'obj'),
+        'application':  ObjType(_('application'),   'app', 'obj'),
     }
 
     directives = {
@@ -584,7 +630,8 @@ class MATLABDomain(Domain):
         'currentmodule':   MatCurrentModule,
         'decorator':       MatDecoratorFunction,
         'decoratormethod': MatDecoratorMethod,
-        'script':            MatModulelevel,
+        'script':          MatModulelevel,
+        'application':     MatModulelevel,
     }
     roles = {
         'data':  MatXRefRole(),
@@ -596,7 +643,8 @@ class MATLABDomain(Domain):
         'meth':  MatXRefRole(fix_parens=True),
         'mod':   MatXRefRole(),
         'obj':   MatXRefRole(),
-        'scpt':   MatXRefRole(),
+        'scpt':  MatXRefRole(),
+        'app':   MatXRefRole(),
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
@@ -607,7 +655,7 @@ class MATLABDomain(Domain):
     ]
 
     def clear_doc(self, docname):
-        for fullname, (fn, _) in list(self.data['objects'].items()):
+        for fullname, (fn, _) in list(self.data['objects'].items()):  # noqa: 401
             if fn == docname:
                 del self.data['objects'][fullname]
         for modname, (fn, _, _, _) in list(self.data['modules'].items()):
@@ -660,15 +708,15 @@ class MATLABDomain(Domain):
             elif modname and modname + '.' + name in objects:
                 newname = modname + '.' + name
             elif modname and classname and \
-                     modname + '.' + classname + '.' + name in objects:
+                    modname + '.' + classname + '.' + name in objects:
                 newname = modname + '.' + classname + '.' + name
             # special case: builtin exceptions have module "exceptions" set
             elif type == 'exc' and '.' not in name and \
-                 'exceptions.' + name in objects:
+                    'exceptions.' + name in objects:
                 newname = 'exceptions.' + name
             # special case: object methods
             elif type in ('func', 'meth') and '.' not in name and \
-                 'object.' + name in objects:
+                    'object.' + name in objects:
                 newname = 'object.' + name
         if newname is not None:
             matches.append((newname, objects[newname]))
@@ -713,18 +761,63 @@ class MATLABDomain(Domain):
         for refname, (docname, type) in self.data['objects'].items():
             yield (refname, refname, type, docname, refname, 1)
 
+
 def setup(app):
     app.add_domain(MATLABDomain)
     # autodoc
     app.add_config_value('matlab_src_dir', None, 'env')
-    app.add_autodocumenter(doc.MatModuleDocumenter)
+    app.add_config_value('matlab_src_encoding', None, 'env')
+    app.add_config_value('matlab_keep_package_prefix', True, 'env')
+
+    app.registry.add_documenter('mat:module', doc.MatModuleDocumenter)
+    app.add_directive_to_domain('mat',
+                                'automodule',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:function', doc.MatFunctionDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autofunction',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:class', doc.MatClassDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autoclass',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:method', doc.MatMethodDocumenter)
+    app.add_directive_to_domain('mat',
+                                'automethod',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:script', doc.MatScriptDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autoscript',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:exception', doc.MatExceptionDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autoexception',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:attribute', doc.MatAttributeDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autoattribute',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:data', doc.MatDataDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autodata',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:instanceattribute', doc.MatInstanceAttributeDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autoinstanceattribute',
+                                mat_directives.MatlabAutodocDirective)
+
+    app.registry.add_documenter('mat:application', doc.MatApplicationDocumenter)
+    app.add_directive_to_domain('mat',
+                                'autoapplication',
+                                mat_directives.MatlabAutodocDirective)
+
     app.add_autodoc_attrgetter(doc.MatModule, doc.MatModule.getter)
-    app.add_autodocumenter(doc.MatClassDocumenter)
-    app.add_autodocumenter(doc.MatExceptionDocumenter)
-    app.add_autodocumenter(doc.MatDataDocumenter)
     app.add_autodoc_attrgetter(doc.MatClass, doc.MatClass.getter)
-    app.add_autodocumenter(doc.MatFunctionDocumenter)
-    app.add_autodocumenter(doc.MatMethodDocumenter)
-    app.add_autodocumenter(doc.MatAttributeDocumenter)
-    app.add_autodocumenter(doc.MatInstanceAttributeDocumenter)
-    app.add_autodocumenter(doc.MatScriptDocumenter)
